var Promise = require('when/es6-shim/Promise');
var underscore = require('underscore')

var Engine = {
	_callback: {},
	_job: [],
	_config: {
		max_job: 10
	},
	_current_job: 0
};

function setJobCount(num){
	this._config.max_job = num;
}

function registerFunction(name, func) {
	if (this._callback[name]) {
		throw new Error("handler for " + name + " is already defined")
	}
	if (!underscore.isFunction(func)) {
		throw new Error("func is not a function")
	}
	this._callback[name] = func
}

function registerAction(name, argList) {
	if (!this._callback[name]) {
		throw new Error("handler for " + name + " is not defined")
	}
	if(!argList){
		argList = []
	}
	var that = this;
	var pr = new Promise(function(resolve,reject) {
		that._job.push({
			name 	: name,
			data 	: argList,
			resolve : resolve,
			reject	: reject
		})
	})
	execJob();
	return pr;
}

function up() {
	Engine._current_job += 1;
}

function down() {
	
	// console.log("Remaining job count",Engine._job.length)
	Engine._current_job -= 1;
	execJob()
}

function execJob() {
	var execCount = Engine._current_job;
	var maxJobCount = Engine._config.max_job;
	//console.log(Engine._current_job)
	if (execCount < maxJobCount && Engine._job.length) {
		up();
		var job = Engine._job.shift();
		var funcName = job.name;
		var funcArgs = job.data;
		var funcHandler = Engine._callback[funcName];
		var resolve = job.resolve;
		var reject = job.reject;
		var ret = funcHandler.apply(null, funcArgs);
		if (typeof ret.then == "function") {
			ret.then(function(result) {
				resolve(result)
				down()
			}, function(reason) {
				reject(reason)
				down()
			}).catch(function(err) {
				reject(err)
				down()
			})
		} else {
			resolve(ret);
			down();
		}
	} else {
		// execJob();
	}
}

Engine.setJobCount = setJobCount;
Engine.registerFunction = registerFunction;
Engine.registerAction = registerAction;

module.exports = Engine;